#include "BuiltIns.h"
#include "ifi_default.h"
#include "ifi_aliases.h"
#include "ifi_utilities.h"
#include "string.h"
#include "my_types.h"
#include "lcd.h"

#include "API.h"

/*  
Name: WPIKeypad.c
Description:
Vex Keypad Experiment 
In this experiment we connect a keypad to the Vex Controller and use it as a simple numeric data entry device or to 
input menu selections for an LCD based User Interface.  
 
We start out just transfering data a byte at a time via the 8-Bit Parallel Interface.

This function must be here for a competition project. It 
is automatically referenced by WPILib at startup and run.  
At that point the SetCompetitionMode function sets the  
competition mode. Basically, a mode of 0 is the default  
(without the IO_Initialization function) and runs a main  
function only.  SetCompetitionMode(1) runs a competition project as shown. 
#define PORT_1 1
#define PORT_2 2
#define CHANNEL_1 1
#define CHANNEL_2 2
#define CHANNEL_3 3
#define CHANNEL_4 4
#define CHANNEL_5 5
#define CHANNEL_6 6
#define CHANNEL_5_TOP 1
#define CHANNEL_5_BOTTOM 2
#define CHANNEL_6_TOP 3
#define CHANNEL_6_BOTTOM 4

// Used in User_Initialization routine in user_routines.c file. 
// Used to set pins as INPUTS (analog or digital) or OUTPUTS (digital only). 
#define INPUT   1
#define OUTPUT  0

/* 
// Used in User_Initialization routine in user_routines.c file. 
// Used to set pins as INPUTS (analog or digital) or OUTPUTS (digital only). 
#define IO1             TRISAbits.TRISA0
#define IO2             TRISAbits.TRISA1
#define IO3             TRISAbits.TRISA2
#define IO4             TRISAbits.TRISA3
#define IO5             TRISAbits.TRISA5
#define IO6             TRISFbits.TRISF0
#define IO7             TRISFbits.TRISF1
#define IO8             TRISFbits.TRISF2
#define IO9             TRISFbits.TRISF3
#define IO10            TRISFbits.TRISF4
#define IO11            TRISFbits.TRISF5
#define IO12            TRISFbits.TRISF6
#define IO13            TRISHbits.TRISH4
#define IO14            TRISHbits.TRISH5
#define IO15            TRISHbits.TRISH6
#define IO16            TRISHbits.TRISH7

// Aliases used to read the pins when used as INPUTS. 
#define rc_dig_in01     PORTAbits.RA0
#define rc_dig_in02     PORTAbits.RA1
#define rc_dig_in03     PORTAbits.RA2
#define rc_dig_in04     PORTAbits.RA3
#define rc_dig_in05     PORTAbits.RA5
#define rc_dig_in06     PORTFbits.RF0
#define rc_dig_in07     PORTFbits.RF1
#define rc_dig_in08     PORTFbits.RF2
#define rc_dig_in09     PORTFbits.RF3
#define rc_dig_in10     PORTFbits.RF4
#define rc_dig_in11     PORTFbits.RF5
#define rc_dig_in12     PORTFbits.RF6
#define rc_dig_in13     PORTHbits.RH4
#define rc_dig_in14     PORTHbits.RH5
#define rc_dig_in15     PORTHbits.RH6
#define rc_dig_in16     PORTHbits.RH7

// Aliases used to drive the pins when used as OUTPUTS. 
#define rc_dig_out01    LATAbits.LATA0
#define rc_dig_out02    LATAbits.LATA1
#define rc_dig_out03    LATAbits.LATA2
#define rc_dig_out04    LATAbits.LATA3
#define rc_dig_out05    LATAbits.LATA5
#define rc_dig_out06    LATFbits.LATF0
#define rc_dig_out07    LATFbits.LATF1
#define rc_dig_out08    LATFbits.LATF2
#define rc_dig_out09    LATFbits.LATF3
#define rc_dig_out10    LATFbits.LATF4
#define rc_dig_out11    LATFbits.LATF5
#define rc_dig_out12    LATFbits.LATF6
#define rc_dig_out13    LATHbits.LATH4
#define rc_dig_out14    LATHbits.LATH5
#define rc_dig_out15    LATHbits.LATH6
#define rc_dig_out16    LATHbits.LATH7

// *
// *-------------------------------------------------------------------------------
// *---------- Aliases for INTERRUPTS connectors (inputs) -------------------------
// *-------------------------------------------------------------------------------
// *  Below are aliases for the input pins labelled INTERRUPTS on the VEX Robot Controller.
// *  These pins are connected to PULL-UPS, meaning they are normally HIGH (1).
// 
#define rc_dig_int1     PORTBbits.RB2
#define rc_dig_int2     PORTBbits.RB3
#define rc_dig_int3     PORTBbits.RB4
#define rc_dig_int4     PORTBbits.RB5
#define rc_dig_int5     PORTBbits.RB6
#define rc_dig_int6     PORTBbits.RB7


*/ 

/*
// Hardware interface with the  Vex Controller (using Microchip PIC18F8520 Pin numbering)
// 8-Bit Parallel Port Control signals
#define CS_BAR_TRIS             TRISFbits.TRISF9
#define WR_BAR_TRIS             TRISFbits.TRISF10
#define RD_BAR_TRIS             TRISFbits.TRISF11
#define BUSY_BAR_TRIS          TRISFbits.TRISF5

#define CS_BAR        rc_dig_out09      // Bits are shifted out this pin # to the MAX7219.
#define WR_BAR        rc_dig_out10      // Data valid on rising edge of this clock (pin RD5).
#define RD_BAR        rc_dig_out11      // This control pin tells the 7219 to transfer data to LEDs.  
#define BUSY_BAR          		PORTFbits.RF5



#define RD_BAR             		LATFbits.LATF3
#define WR_BAR             		LATFbits.LATF4
#define BUSY_BAR          		PORTFbits.RF5
*/

//*****************************************************************************************************
// Define keypad constants here...
#define LOW  0                                  // Logic low
#define HIGH 1                                  // Logic high

// Hardware interface with the  Vex Controller (using Vex Controller Pin numbering)
 
#define CS_BAR           9      			// Chip Select to enable the peripheral controller parallel interface
#define WR_BAR        	10      			// Write to the peripheral controller
#define RD_BAR       	11      			// Read from the peripheral controller
#define BUSY_BAR        12      			// Signal indicating that the peripheral is ready to accept another parallel command

//*****************************************************************************************************
 
// Define data structure for virtual 8-Bit Port.  This can be 
// extended to 16-Bit or 32-Bit virtual ports similar to the BS2/BSX Stamp
enum Port_T {PORT_A, PORT_B, PORT_C, PORT_D, PORT_E, PORT_F, PORT_G, PORT_H, PORT_J}; 

typedef volatile union  {
    byte Port_Value;		// The port value as a byte
    struct {
		// Copy the virtual port bits to the physical port bit assignments using  MSB first (Big Endian?)
        byte VR_BIT_7:1;
        byte VR_BIT_6:1;
        byte VR_BIT_5:1;
        byte VR_BIT_4:1;
        byte VR_BIT_3:1;
        byte VR_BIT_2:1;
        byte VR_BIT_1:1;
        byte VR_BIT_0:1;
/*
		// Copy the virtual port bits to the physical port bit assignments using LSB first (Little Endian?)
        byte VR_BIT_0:1;
        byte VR_BIT_1:1;
        byte VR_BIT_2:1;
        byte VR_BIT_3:1;
        byte VR_BIT_4:1;
        byte VR_BIT_5:1;
        byte VR_BIT_6:1;
        byte VR_BIT_7:1;  
*/   
    } ;
 
} VR_8_BIT_PORT_T;   

// Declare local function headers here
VR_8_BIT_PORT_T Read_Virtual_Port(byte Port_ID);

VR_8_BIT_PORT_T VR_Port_Data;	// Virtual port data (8-Bits)

void Write_Virtual_Port(byte Port_ID, VR_8_BIT_PORT_T The_Data);
byte Read_Byte(void);

// Buffer used to format output to the LCD
char Buffer[80];
const rom int PowerOfTen[3] = {100,10,1};

// Messages and labels used for the VEX User Interface (UI)
rom char Directionlabel[] = "Enter Direction (0=CCW, 1=CW): \0";
rom char MotorIDLabel[] = "Enter Motor ID (1..8): \0";
rom char MotorSpeedLabel[] = "Enter Motor Speed (0..255): \0";
rom char ServoIDLabel[] = "Enter Servo ID: (1..8): \0";


/***************************************************************************************
 * IO_Initialization -  This procedure initializes the Vex Controller and I/O Ports. 
 ***************************************************************************************/
void IO_Initialization(void)
{ 
	SetCompetitionMode(0,1); // VEX Runs main
  //SetCompetitionMode(1, 1); // VEX
 
   //Controller Port   #: 01,02,03,04,05,06,07,08,09,10,11,12,13,14,15,16 
   //DefineControllerIO(4, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0); 
   // The first parameter is the number of analog input ports.
   //                        01     02     03     04     05     06     06     07     08     09      10      11     13      14      15      16
   //DefineControllerIO ( 5, INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,INPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
  // DefineControllerIO ( 2, INPUT ,INPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   //DefineControllerIO (   0, INPUT ,INPUT ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
   DefineControllerIO (   0, OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,INPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
 
} 
 

/***************************************************************************************
 *  Initialize - This function is run immediately when the robot is powered on 
 *  regardless of the field mode. 
 ***************************************************************************************/
void Initialize(void) 
{ 
	int i; 
	for (i = 0; i < 10; i++) 
	{ 
		PrintToScreen("Initialize %d\r", i); 
		//printf("Initialize %d\r", i); 
		Wait(500); 
	} 
} 


 
//******************************************************************************
//* Read_Virtual_Port - Read a byte of data from the selected Port,  
//* Port_ID range (PORT_A..PORTE).  It reads the bits from the
//* assigned ports and saves them to Virtual Port A, bits A(0:5), Port F bits(0:2)  
//*   
//******************************************************************************

VR_8_BIT_PORT_T Read_Virtual_Port(byte Port_ID)
{
	VR_8_BIT_PORT_T The_Virtual_Port;

	switch(Port_ID)
	{
		case PORT_A:
		{
/*
			// Set PORT A bits to inputs
			TRISAbits.TRISA0 = 1;
			TRISAbits.TRISA1 = 1;
			TRISAbits.TRISA2 = 1;
			TRISAbits.TRISA3 = 1;
			TRISAbits.TRISA5 = 1;

			// Set PORT F bits to inputs
			TRISFbits.TRISF0 = 1;
			TRISFbits.TRISF1 = 1;
			TRISFbits.TRISF2 = 1;
*/
            //DefineControllerIO (   0, INPUT ,INPUT ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,INPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
            IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO8 = INPUT;

			// Copy the virtual port bits to the physical port bit assignments using  MSB first (Big Endian?)
            The_Virtual_Port.VR_BIT_7 = PORTAbits.RA0;
		 	The_Virtual_Port.VR_BIT_6 = PORTAbits.RA1;;
            The_Virtual_Port.VR_BIT_5 = PORTAbits.RA2;
		 	The_Virtual_Port.VR_BIT_4 = PORTAbits.RA3;
            The_Virtual_Port.VR_BIT_3 = PORTAbits.RA5;;
		 	The_Virtual_Port.VR_BIT_2 = PORTFbits.RF0;
		 	The_Virtual_Port.VR_BIT_1 = PORTFbits.RF1;
		 	The_Virtual_Port.VR_BIT_0 = PORTFbits.RF2;

#ifdef NEW_CODE
            
			// Copy the virtual port bits to the physical port bit assignments using LSB first (Little Endian?)
            The_Virtual_Port.VR_BIT_0 = PORTAbits.RA0;
		 	The_Virtual_Port.VR_BIT_1 = PORTAbits.RA1;;
            The_Virtual_Port.VR_BIT_2 = PORTAbits.RA2;
		 	The_Virtual_Port.VR_BIT_3 = PORTAbits.RA3;
            The_Virtual_Port.VR_BIT_4 = PORTAbits.RA5;;
		 	The_Virtual_Port.VR_BIT_5 = PORTFbits.RF0;
		 	The_Virtual_Port.VR_BIT_6 = PORTFbits.RF1;
		 	The_Virtual_Port.VR_BIT_7 = PORTFbits.RF2;
#endif
			break;
		}

		case PORT_D:
		{
			The_Virtual_Port.Port_Value = PORTD;
			break;
		}

	}
	return The_Virtual_Port;	
}


//******************************************************************************
//* Write_Virtual_Port - Write a byte of data from the selected port, 
//* Port_ID range (PORT_A..PORTE). It Distributes the bits from the virtual data byte to the
//* Port_ID range (PORT_A..PORTE) using the following
//* assigned ports and saves them to Virtual Port A, bits A(0:5), Port F bits(0:2)  
//******************************************************************************
void Write_Virtual_Port(byte Port_ID, VR_8_BIT_PORT_T The_Data)
{
	// Distribute the bits from the virtual data byte to the
	// assigned for Virtual Port A, bits to Virtual Port A, bits A(0:5), Port F bits(0:2) 
	switch(Port_ID)
	{
		case PORT_A:
		{
/*
			// Set PORT A bits to outputs
			TRISAbits.TRISA0 = 0;
			TRISAbits.TRISA1 = 0;
			TRISAbits.TRISA2 = 0;
			TRISAbits.TRISA3 = 0;
			TRISAbits.TRISA5 = 0;

			// Set PORT F bits to outputs
			TRISFbits.TRISF0 = 0;
			TRISFbits.TRISF1 = 0;
			TRISFbits.TRISF2 = 0;
*/
            //DefineControllerIO (   0, OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT  ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ,OUTPUT ) ;
            IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO8 = OUTPUT;

			// Copy the virtual port bits to the physical port bit assignments using MSB first (Big Endian?)
            LATAbits.LATA0 = The_Data.VR_BIT_7;
			LATAbits.LATA1 = The_Data.VR_BIT_6;
			LATAbits.LATA2 = The_Data.VR_BIT_5;
			LATAbits.LATA3 = The_Data.VR_BIT_4;
			LATAbits.LATA5 = The_Data.VR_BIT_3;
			LATFbits.LATF0 = The_Data.VR_BIT_2;
			LATFbits.LATF1 = The_Data.VR_BIT_1;
			LATFbits.LATF2 = The_Data.VR_BIT_0;

#ifdef NEW_CODE
            
			// Copy the virtual port bits to the physical port bit assignments using LSB first (Little Endian?)
            LATAbits.LATA0 = The_Data.VR_BIT_0;
			LATAbits.LATA1 = The_Data.VR_BIT_1;
			LATAbits.LATA2 = The_Data.VR_BIT_2;
			LATAbits.LATA3 = The_Data.VR_BIT_3;
			LATAbits.LATA5 = The_Data.VR_BIT_4;
			LATFbits.LATF0 = The_Data.VR_BIT_5;
			LATFbits.LATF1 = The_Data.VR_BIT_6;
			LATFbits.LATF2 = The_Data.VR_BIT_7;
#endif
			break;
		}
 
		case PORT_D:
		{
			// Set PORT D bits to outputs
			TRISD = 0;

			PORTD = The_Data.Port_Value;

			break;
		}

	}
}


 
//******************************************************************************************
//* main -  The main program is used to generate a test ramp 0..255 and
//* send it to the controller.
//****************************************************************************************** 
void main(void)
{ 	
	unsigned char port;
	unsigned char function;
	unsigned char pwm;
	unsigned char invert;
    unsigned char LeftPWM;
    unsigned char RightPWM;
    unsigned int Joy_X_1;
    unsigned int Joy_Y_1;
    unsigned int Joy_X_2;
    unsigned int Joy_Y_2;
	int i;
    int Data_Byte = 0;
    int KeyValue = 0;
    int Direction = 0; // Default to Counter Clockwise (CCW) motor direction (CCW=0, CW=1)
    int Speed = 0;     // Motor speed (0..120 = CCW; 120..128 = Stop; 129..255 = CW) - These values may need to be calibrated!

	// Open serial ports here
	//OpenSerialPortOne(BAUD_9600);
	//OpenSerialPortTwo(BAUD_9600);
	OpenSerialPortOne(BAUD_115200);
	OpenSerialPortTwo(BAUD_115200);

    // Set digital I/O port directions and initial values
    IO_Initialization();


    //IO9 = 0;    // RD_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       
    //IO10 = 0;   // WR_BAR_TRIS = 0;     	// Set RD_BAR pin direction to output              

    //IO11 = 1;   // BUSY_BAR_TRIS = 1;    	// Set RD_BAR pin direction to input         

    //BUSY_BAR_TRIS = 1;    	// Set BUSY_BAR pin direction to input         
    //RD_BAR_TRIS = 0;        // Set RD_BAR pin direction to output       
    //WR_BAR_TRIS = 0;        // Set WR_BAR pin direction to output       
    //CS_BAR_TRIS = 0;        // Set CS_BAR pin direction to output       

    // Set Vex Controller I/O Directions
	// Set the Vex Controller Digital I/O to Outputs assignments here...
	SetDirection(RD_BAR, OUTPUT);             			// Direction for left stepper motor
	SetDirection(WR_BAR, OUTPUT);     						// Used to activate left stepper motor (pulse)
	SetDirection(CS_BAR, OUTPUT);       					// Selects half step sequence (high)

 
	SetDirection(BUSY_BAR, INPUT);     					// Selects one phase wave drive (high) or two phase (low)

     // Start with high levels 
	SetDigitalOutput(RD_BAR, HIGH);
	SetDigitalOutput(WR_BAR, HIGH);
	SetDigitalOutput(CS_BAR, HIGH);

    //Test parallel port control signals.  Works!!!
    //SetupTimers();  
    PrintToScreen("\r\n Initializing the Vex Keypad Interface... \r\n"); 
/*    
    // Send test messages to the serial terminal
    while(1)
    {
        for (i=0; i<100; i++)
     	{
           PrintToScreen("\r\n This is a test %d times \r\n", i); 		
     	}
    }
*/

 //Format the Message to be displayed on the LCD 

    Buffer[0] = ' ';
    Buffer[1] = 'E';
    Buffer[2] = 'n';
    Buffer[3] = 't';
    Buffer[4] = 'e';
    Buffer[5] = 'r';
    Buffer[6] = ' ';
    Buffer[7] = 'S';
    Buffer[8] = 'p';
    Buffer[9] = 'e';
    Buffer[10] = 'e';
    Buffer[11] = 'd';
    Buffer[12] = '(';
    Buffer[13] = '0';
    Buffer[14] = '.';
    Buffer[15] = '.';
    Buffer[16] = '2';
    Buffer[17] = '5';
    Buffer[18] = '5';
    Buffer[19] = ')';
    Buffer[20] = ':';      
    Buffer[21] = 0;     // String terminator (null character)  

    // Copy message from Flash to RAM
   //strcpypgm2ram (Buffer,(rom char *)"This is a test!!!");  
   //strcpypgm2ram (Buffer,(rom char *) MotorSpeedLabel);  
   //len = (int) strlen((const char *)ifi_printfBufr);
   //if (len > 80) len = 80;

    // Position the cursor to the first line
    lcd_goto(0,2);

    Wait(1);

    // Send message to the LCD Display
    lcd_puts((char *) Buffer);  
	 while (1)   // Forever
  	{
        lcd_putc('A');
        lcd_putc('B');
        lcd_putc('C');
        lcd_putc('D');
        lcd_putc('E');
        lcd_putc('F');

 		for (i=0; i<255; i++)
		{
			PrintToScreen("Testing %d times!!!\n", i);
			Wait(500);
			
		}
    }

   // Configure ports and initialize variables used by the Keypad application
    InitializeKeypad();

    // Demo loop which reads values from keypad and sends them to the host. It
    // Waits for press to indicate a keypress, then displays the key on the debug
    // screen.  Note that this code clears the press bit when done in order to
    // prepare for the next press.

   
    while(1)
    {
    
    	//KeyValue = GetKey() - 48;			// Convert the key value from ASCII to binary
    	KeyValue = GetKey();				// Convert the key value from ASCII to binary
  
    	// Send the scanned key to the host

    	PrintToScreen("Key pressed = %d \r\n", KeyValue);
   	 

    }



    for (i=0; i<100; i++)
    {
       PrintToScreen("This is a test %d times \r\n", i); 
       //Wait(100};
    }
 
    Wait(1000);

     // Initialize the LCD Display
	InitializeLCD();
  //Format the Message to be displayed on the LCD 
    Buffer[0] = ' ';
    Buffer[1] = 'E';
    Buffer[2] = 'n';
    Buffer[3] = 't';
    Buffer[4] = 'e';
    Buffer[5] = 'r';
    Buffer[6] = ' ';
    Buffer[7] = 'S';
    Buffer[8] = 'p';
    Buffer[9] = 'e';
    Buffer[10] = 'e';
    Buffer[11] = 'd';
    Buffer[12] = '(';
    Buffer[13] = '0';
    Buffer[14] = '.';
    Buffer[15] = '.';
    Buffer[16] = '2';
    Buffer[17] = '5';
    Buffer[18] = '5';
    Buffer[19] = ')';
    Buffer[20] = ':';      
    Buffer[21] = 0;     // String terminator (null character)  

   
    // Position the cursor to the first line
    lcd_goto(0,2);

    Wait(1);

    // Send message to the LCD Display
    lcd_puts((char *) Buffer);  

   // Send message to the Serial Terminal if available
  	printf("Enter Speed (0..255) \r\n", KeyValue);
    //Format the Message to be displayed on the LCD 
    Buffer[0] = 'S ';
    Buffer[1] = 'p';
    Buffer[2] = 'e';
    Buffer[3] = 'e';
    Buffer[4] = 'd';
    Buffer[5] = ' ';
    Buffer[6] = '=';
    Buffer[7] = ' ';
    Buffer[8] = 0;     // String terminator (null character)
 
  

    Speed = 0;
    i = 0;
    while(1)
    {
 
    	//KeyValue = GetKey() - 48;			// Convert the key value from ASCII to binary
    	KeyValue = GetKey();				// Convert the key value from ASCII to binary

        // Check the range of the Key Value to make sure is a digit between 0 and 9
        if ((KeyValue >=0) && (KeyValue <=9))
        {
           // Enter the Speed one digit at a time
           Speed += (int)KeyValue*PowerOfTen[i];
           i++;
    
           // Position the cursor
           lcd_goto(1,2);
           //Wait(1);

           // Display formatted text to the LCD
    	   lcd_puts((char *) Buffer);  
           lcd_printdec(Speed);
 
    	   // Send the scanned key to the host serial terminal also
       	   printf("Speed = %d \r\n", KeyValue);
 
   	       // Send the scanned key to the host serial terminal also
       	   printf("Speed = %d \r\n", Speed);

           if (i > 2)
           {
              if ( Speed < 256)
              {

                 // Set the VEX Motor # 1 Speed 
                 pwm01 = Speed;

                 // Reset digit count and Speed
                 i = 0;
                 Speed = 0;
              }

              // Reset digit count and Speed
              i = 0;
              Speed = 0;
           }
        }
   	 
    }

    // Test 2 - Display each kystroke entered from the keypad to the LCD
    // Clear the LCD and move the cursor to home position
    lcd_clear();
  
     //Format the Message to be displayed on the LCD 
    Buffer[0] = ' ';
    Buffer[1] = 'E';
    Buffer[2] = 'n';
    Buffer[3] = 't';
    Buffer[4] = 'e';
    Buffer[5] = 'r';
    Buffer[6] = ' ';
    Buffer[7] = 'K';
    Buffer[8] = 'e';
    Buffer[9] = 'y';
    Buffer[10] = ' ';
    Buffer[11] = '(';
    Buffer[12] = '0';
    Buffer[13] = '.';
    Buffer[14] = '.';
    Buffer[15] = '9';
    Buffer[16] = ')';
    Buffer[17] = ':';      
    Buffer[18] = 0;     // String terminator (null character)
   
    // Position the cursor to the first line
    lcd_goto(0,2);

    Wait(1);

    // Send message to the LCD Display
    lcd_puts((char *) Buffer);  

   // Send message to the Serial Terminal if available
  	printf("Enter Key ('0'..'9') \r\n", KeyValue);
 
  //Format the Message to be displayed on the LCD 
    Buffer[0] = ' ';
    Buffer[1] = 'K';
    Buffer[2] = 'e';
    Buffer[3] = 'y';
    Buffer[4] = ' ';
    Buffer[5] = 'p';
    Buffer[6] = 'r';
    Buffer[7] = 'e';
    Buffer[8] = 's';
    Buffer[9] = 's';
    Buffer[10] = 'e';
    Buffer[11] = 'd';
    Buffer[12] = ' ';
    Buffer[13] = '=';
    Buffer[14] = ' ';
    Buffer[15] = 0;     // String terminator (null character)
    while(1)
    {
 
    	//KeyValue = GetKey() - 48;			// Convert the key value from ASCII to binary
    	KeyValue = GetKey();				// Convert the key value from ASCII to binary

        // Check the range of the Key Value to make sure is a digit between 0 and 9
        if ((KeyValue >=0) && (KeyValue <=9))
        {
           // Position the cursor
           lcd_goto(1,2);
           Wait(1);

           // Display formatted text to the LCD
    	   lcd_puts((char *) Buffer);  
           lcd_printdec(KeyValue);
 
    	   // Send the scanned key to the host serial terminal also
       	   printf("Key pressed = %d \r\n", KeyValue);
        }
   	 
    }


 
 
/*
	 while (1)   // Forever
  	{
 		for (i=0; i<255; i++)
		{
			printf("Testing %d times!!!\n", i);
		    Wait(500);
			
		}
    }
*/

  //Format the Message to be displayed on the LCD 
/*
    Buffer[0] = ' ';
    Buffer[1] = 'E';
    Buffer[2] = 'n';
    Buffer[3] = 't';
    Buffer[4] = 'e';
    Buffer[5] = 'r';
    Buffer[6] = ' ';
    Buffer[7] = 'S';
    Buffer[8] = 'p';
    Buffer[9] = 'e';
    Buffer[10] = 'e';
    Buffer[11] = 'd';
    Buffer[12] = '(';
    Buffer[13] = '0';
    Buffer[14] = '.';
    Buffer[15] = '.';
    Buffer[16] = '2';
    Buffer[17] = '5';
    Buffer[18] = '5';
    Buffer[19] = ')';
    Buffer[20] = ':';      
    Buffer[21] = 0;     // String terminator (null character)  
*/
    // Copy message from Flash to RAM
   //strcpypgm2ram (Buffer,(rom char *)"This is a test!!!");  
   //strcpypgm2ram (Buffer,(rom char *) MotorSpeedLabel);  
   //len = (int) strlen((const char *)ifi_printfBufr);
   //if (len > 80) len = 80;

    // Position the cursor to the first line
    lcd_goto(0,2);

    Wait(1);

    // Send message to the LCD Display
    lcd_puts((char *) Buffer);  
	 while (1)   // Forever
  	{
        lcd_putc('A');
        lcd_putc('B');
        lcd_putc('C');
        lcd_putc('D');
        lcd_putc('E');
        lcd_putc('F');

 		for (i=0; i<255; i++)
		{
			printf("Testing %d times!!!\n", i);
			Wait(500);
			
		}
    }


    PrintToScreen("\r\n Enter keystroke using keypad... \r\n"); 

    // Configure ports and initialize variables used by the Keypad application
    InitializeKeypad();

    // Demo loop which reads values from keypad and sends them to the host. It
    // Waits for press to indicate a keypress, then displays the key on the debug
    // screen.  Note that this code clears the press bit when done in order to
    // prepare for the next press.

   
    while(1)
    {
    
    	//KeyValue = GetKey() - 48;			// Convert the key value from ASCII to binary
    	KeyValue = GetKey();				// Convert the key value from ASCII to binary
  
    	// Send the scanned key to the host

    	PrintToScreen("Key pressed = %d \r\n", KeyValue);
   	 

    }



    for (i=0; i<100; i++)
    {
       PrintToScreen("This is a test %d times \r\n", i); 
       //Wait(100};
    }
 
    Wait(1000);

   

	// This code works!!!
  	TwoWheelDrive(1, 2); // define robot two wheels 

	while (1)
	{
		// Get data from first joystick (right)  Works!!!
  		Joy_X_1 = GetOIAInput(PORT_2, CHANNEL_1);
		Joy_Y_1 = GetOIAInput(PORT_2, CHANNEL_2);

		// Get data from second joystick (left)  Works!!!
   		Joy_X_2 = GetOIAInput(PORT_2, CHANNEL_3);
		Joy_Y_2 = GetOIAInput(PORT_2, CHANNEL_4);

    	PrintToScreen("Joy_X_1 = %d, Joy_Y_1 = %d, Joy_X_2 = %d, Joy_Y_2 = %d \r", Joy_X_1, Joy_Y_1, Joy_X_2, Joy_Y_2 ); 
		//Wait(500);


		// Move the selected motors.  Works!!!
		SetPWM (1,Joy_X_1);
		SetPWM (2,Joy_Y_1);		
        SetPWM (3,Joy_X_2);
		SetPWM (4,Joy_Y_2);

 /*
		// Move motors using updated joytick Arcade style positions 
		Arcade2(PORT_1,
                CHANNEL_1, CHANNEL_2,
                LeftPWM, RightPWM,
                0, 0);
*/

	}
/*
An earlier search didn't turn up anything on SetMotor() or SetServo() but a second search turned up some examples using SetPWM(). 
I believe my problem has to do with the size of the variable I was using as an argument. Things seem to be working if I use:

unsigned char armASpeed = 0;
while(1) {
armASpeed = GetRxInput(1,3);
PrintToScreen("armASpeed: %d\n",(int)armASpeed);
SetPWM(8, armASpeed);
}

*/
	while(1)
	{
		// Read Vex RC joystick positions 
 		OIToPWM(PORT_1, 1,  LeftPWM , 0);
	    OIToPWM(PORT_1, 2,  RightPWM, 0);
    	PrintToScreen("LeftPWM = %d, RightPWM = %d \r", LeftPWM, RightPWM); 

		// Move motors using updated joytick Arcade style positions 
		Arcade2(PORT_1,
                CHANNEL_1, CHANNEL_2,
                LeftPWM, RightPWM,
                0, 0);
	}


	TwoWheelDrive(1, 2); // define robot two wheels 
	StartUltrasonic(1, 7); // start ultrasonic rangefinder 
	while (GetUltrasonic(1, 7) > 24) // loop until about 2 feet away 
	{ 
		Drive(100, 0); // drive forward } 
		StopUltrasonic(1, 7); // turn off rangefinder 
		Drive(0, 0); // stop driving
	 }
}
