//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  lcd.c                                                             *
//*   Author:  Daniel F. Ramirez                                                 *           
//*     Date:  02/05/2002                                                        *
//*                                                                              *
//* Description:                                                                 *
//*         This is an embedded realtime LCD Display application, that uses the  *
//* LCD library and a PIC18F452 micro-controller from Microchip Technology to    *
//* drive both serial and directly connected LCD displays.  It employs the       *
//* shiftout and pulsout functions to communicate to the Serial LCD and uses     *
//* reverse, itoa and right_justify functions to format data.                    *
//*                                                                              *
//* The basic LCD driver functions were ported from code developed by            *
//* Jon Williams from his Nuts&Volts articles on LCD displays.  Thanks Jon.      *
//* The remaining high level functions were developed by Mark Crosbie.           *
//* I modified this version of the software to run on PICDEMO2 board.            *                              
//*                                                                              *
//* lcd.c Routines for sending data to a serial LCD                              *
//*  This application uses code from Mark Crosbie.  Thanks Jon and Mark.         *
//*  Copyright (C) 1998  Mark Crosbie  mcrosbie@best.com                         *
//*                                                                              *
//*  This program is free software; you can redistribute it and/or modify        *
//*  it under the terms of the GNU General Public License as published by        *
//*  the Free Software Foundation; either version 2 of the License, or           *
//*  (at your option) any later version.                                         *
//*                                                                              *
//*  This program is distributed in the hope that it will be useful,             *
//*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
//*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
//*  GNU General Public License for more details.                                *
//*                                                                              *
//*  You should have received a copy of the GNU General Public License           *
//*  along with this program; if not, write to the Free Software                 *
//*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   *
//*                                                                              *
//*   void lcd_putc(char c)    send the byte c to the LCD                        *
//*   void lcd_clear(void)     clears the LCD screen                             *
//*   void lcd_home(void)      homes the LCD cursor                              *
//*   void lcd_puts()          print a string **UNIMPLEMENTED**                  *
//*   void lcd_printhex(char c) prints byte c as a 2 digit hex character         *
//*   void lcd_printdec(char c) prints byte c as a decimal digit string          *
//*   void lcd_printbin(char c) prints byte c as a binary string                 *
//*   void lcd_scroll_left(char n) scrolls LCD screen left n positions           *
//*   void lcd_scroll_right(char n) scrolls LCD screen right n positions         *
//*   void lcd_goto(char row, char col) goto a position                          *
//*                                                                              *
//* Constants used:                                                              *
//*    LCDPORT        port on the PIC the LCD is connected to                    *
//*    LCDPIN         what PIN the serial LCD input is connected to, when used   *
//*                   with the Scott Edwards serial LCD display.                 *
//*                                                                              *
//* Libraries used:                                                              *
//*    pause() from the timeutil library.                                        * 
//*    pulsout() from the portutil library.                                      *                           
//*                                                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large          *
//* multiline LCD displays, LCD graphics and formatting and displaying data on   * 
//* LCD displays.                                                                *                             
//*                                                                              *
//********************************************************************************
   

  
// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif


#ifdef PIC18F8520   	          // Register definitions for PIC18F8520 used by Vex Controller
//#include <p18f8520.h>
#include "ifi_picdefs.h"  // dfr
#include "ifi_aliases.h"  // dfr
//#include "timeutil.h"                                    // Header file for timer utilities
//#include "stdio.h"
//#include "portutil.h"
//#include "def_printf_lib.h"
#endif

#ifdef PIC18F8720     	// Register definitions for PIC18F8720
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
#include <p18f6620.h>
#endif
 
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//#include <i2c.h>												   // I2C library functions 
#include "lcd.h"                                         // LCD library functions
#include "my_types.h"                    // My user types
#include "strio.h"  // String output routines


#define LCD_TEST_1                                	// Select LCD test #1
#define LCD_TEST_2										  		// Select LCD test #2
#define LCD_TEST_3                                	// Select LCD test #3
//#define LCD_TEST_4										  		// Select LCD test #4

        
// Change these definitions if you move the LCD 

#define LCDPORT PORTA    // Port used by Vex Controller for I01..I04 pins
#define LCDPIN 1

#define LCDCMD 254
#define LCDCLS 1
#define LCDHOME 2
#define LCDLINE1BASE 128
#define LCDLINEINCR 64
#define LCDSCRLRIGHT 28
#define LCDSCRLLEFT 24

//*******************************************************************************

// LCD Connections to PIC I/O Pins:

//

// LCD        (Function)    VEX         	Stamp                  PIC18F8520            BS2/BSX 

// ---------------------    ---         	-----                  --------            -------

// pin 1         Vss                 	Gnd

// pin 2         Vdd                 	+5

// pin 3         Vo                  	wiper of 10K pot

// pin 4         RS      	IO7        	P4     				  		RA5                 3

// pin 5         R/W               	 	Gnd					 		RA2				  	2		

// pin 6         E      	IO6        	P5							RF0                 1

// pin 7         DB0             		Gnd

// pin 8         DB1             		Gnd

// pin 9         DB2             		Gnd

// pin 10        DB3             		Gnd

// pin 11        DB4        IO1    		P0						  	RA0					19					

// pin 12        DB5        IO2     	P1						  	RA1					20

// pin 13        DB6        IO3      	P2 					  	  	RA2					21

// pin 14        DB7        IO4   		P3 					  	  	RA3					22


// -----[ Constants ]-------------------------------------------------------

// These constant port I/O pin labels are usefull for porting code from the Stamp BS2/BSX 
// to the PIC.  They correspond to contiguous I/O addresses for each port so that groups of
// I/O Pins may be addressed correctly in a similar manner to the Stamp.  This allows Stamp 
// instructions such as DIRS, DIRH and DIRL and Outs to be used with the PIC.  The P0..P15 I/O
// addresses have been shifted back to 0 => P0, 1 =>P1, ... ,15 => P15  for the PIC.


//      PIC/Stamp mapping of I/O addresses        
#define P0            0                       // These Pin labels correspond to the Stamp BS2/BSX
#define P1            1                       // For use with BS2 compatible PIC portutil functions.
#define P2            2                       // I may want to move these to portutil.h
#define P3            3 
#define P4            4 
#define P5            5 
#define P6            6 
#define P7            7 
#define P8            8 
#define P9            9 
#define P10          10 
#define P11          11 
#define P12          12 
#define P13          13 
#define P14          14 
#define P15          15 

//#define RS_DIR            IO7                       // LCD Register Select (1 = char).  
#define RS_DIR            IO10                       // LCD Register Select (1 = char).  

//#define E_DIR             IO6                       // LCD Enable pin  (1 = enabled).  
#define E_DIR             IO11                       // LCD Enable pin  (1 = enabled).  

//#define RS   rc_dig_out07
#define RS   rc_dig_out10

//#define E    rc_dig_out06
#define E    rc_dig_out11

// LCD control characters

//

#define ClrLCD        0x01                     // clear the LCD

#define CrsrHm        0x02                     // move cursor to home position

#define CrsrLf        0x10                     // move cursor left

#define CrsrRt        0x14                     // move cursor right

#define DispLf        0x18                     // shift displayed chars left

#define DispRt        0x1C                     // shift displayed chars right

#define DDRam         0x80                     // Display Data RAM control

#define Line1	 	    0x00							  // starting address of line 1

#define Line2	 	    0x40							  // starting address of line 2
 

#define TRUE 1                                          // Logical True value
#define FALSE 0                                         // Logical False value

// ****************** Variables ****************************
//extern int LCD_Ready;   // dfr
int	LCD_Ready = TRUE;
 
char Msg1[]  = "Hello Vex Forum";			// preload EEPROM with messages
char Msg2[]  = "This is a test";		    	// preload EEPROM with messages

  
byte LCDLine = 0; 							// Current LCD Line
byte LCDColumn = 0; 							// Current LCD Column

// ******************* EEPROM Data *************************

//

// Declare Local Function Headers Here
void  string(char *s);
void  send_byte(byte c);

#ifdef DEBUG
void pause(word Delay);
#endif

void pulsout ( int Pin,  int Time );


#ifdef LCD_TEST
//********************************************************************************
//* main - Main LCD application
//********************************************************************************

void main (void)
{

  	byte i;									// Loop index

  	// Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
  	//InitializeUsart();  // dfr
  	
 	//printf("trace 1: Setting up timers... \r\n");
	
 	SetupTimers();                                         // Setup timers and software
                                                          // variables. 	
  	// Initialize the LCD Display 

  	InitializeLCD();
  
  // Test the LCD 
  
  printf("0123456789ABCDEF\r\n");
  pause(5000);
  printf("FEDCAB9876543210\r\n");
  pause(5000);
  
  printf("Line 1\r\n");
  pause(5000);
  
  printf("Line 2\r\n");
  pause(5000);

  for (i=0; i<100; i++)
  {
  	printf("i = ");
  	dec(i);
  	printf("\r\n");
  	printf("   \r\n");
  	pause(1000);
  }  
  
  
  printf("LCD Application ...\r\n");
  printf("Delay 5 seconds ...\r\n");
  pause(5000);
  printf("end of 5 second delay \r\n");
  
  
/*  	
  	printf("trace 0: Testing Port A with 0xAA pattern...\r\n");
  	
  	// Write a test patter to Port A
  	
  	while (1)
  	{
  		PORTDA= 0xAA;
  	}
*/

  	lcd_clear();						// clear LCD, home cursor

	// Test the LCD display routines here

#ifdef LCD_TEST_1
   lcd_test_1();
#endif

#ifdef LCD_TEST_2
   lcd_test_2();
#endif

#ifdef LCD_TEST_3
   lcd_test_3();
#endif

#ifdef LCD_TEST_4
   lcd_test_4();
#endif


}

#endif
 
//********************************************************************************
//* InitializeLCD - Initializes variables and registers used by LCD functions.
//********************************************************************************

void            InitializeLCD(void)
{
	int i;                                                 // Loop index
	

   //printf("InitializeLCD: trace 2: Configuring Port A... \r\n");
   
   // configure port A for all digital inputs

//	OpenADC(ADC_0ANA, ADC_INT_OFF);

// -----[ LCD Initialization ]--------------------------------------------------

 
 
	// Initialize an LCD that is directly connected to the PIC micro-controller here

Init:   

	//TRISA = 0x00;                      	// Set LCD control pins 0-2 as outputs (Port E)
										// Output = 0 in PIC, 1 for Stamp BS2/BSX
    //TRISD = 0x00;           			// Set LCD data bits to output (PORTD)           

    // Set IO Bits to outputs
    IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = IO7 = IO10 = IO11 = 0;
	RS_DIR = 0;
	E_DIR = 0;

    // Initialize the ports used for LCD
    PORTA = 0x00;
 
    // Initialize port I/O direction
    TRISA = 0x00;
 

/*
   // Test code to generate pulses from each LCD line 
   printf("trace 0: I/O test \r\n");
   
   for (i=0; i<9000; i++)
   {
   	pulsout(P0,1);
		pulsout(P1,1);
		pulsout(P2,1);
		pulsout(P3,1);
		pulsout(RS,1);
		pulsout(E,1);
   	pause(20);
   }
   printf("trace 1: End of I/O Test.\r\n");
*/   

	// Outs = 0x0000                    // clear the pins
	PORTA = 0x00;                     	// Clear the LCD Control pins
     
	// Initialize the LCD (Hitachi HD44780 controller)

 	pause(500);                      	// let the LCD settle

  	PORTA = 0b0011;							// 8-bit mode
 
	pulsout( E, 1);
	pause(5);
	
  	PORTA = 0b0011;							// 8-bit mode
	pulsout(E, 1);
 	
  	PORTA = 0b0011;							// 8-bit mode	
	pulsout(E, 1);
 	
	//OutA = %0010                    	// 4-bit mode
	PORTA = 0b0010;                    	// 4-bit mode
 
	pulsout(E, 1);
 
   //printf("trace 4: \r\n");
   
	WriteLCDCommand(0x28);              // 2-line mode
                  
	WriteLCDCommand(0x0C);              // disp on, crsr off, blink off                     

	WriteLCDCommand(0x06);              // inc crsr, no disp shift

    pause(5);

	LCD_Ready = TRUE;
		 
	// Now all the high level LCD routines should work !

    lcd_clear();
    
    
} 
 
//********************************************************************************
//* WriteLCDCommand - Send a command to the Hitachi HD44780 LCD display controller
//********************************************************************************

void WriteLCDCommand(byte Command)
{

   byte LowNibble, HighNibble;

	RS = 0;                       	// Enter command mode
												// then write the character
    
	// Write an ASCII character to the LCD 

   HighNibble = (Command >> 4) & 0x0F;
	PORTA =  HighNibble ;            // Output high nibble (leave other Port D bits alone)

	pulsout(E, 1);                    	// strobe the Enable line
	
   //pause(1);
   LowNibble = Command & 0x0F;
	PORTA = LowNibble;             // output low nibble (leave other Port D bits alone)

	pulsout(E, 1);
   //pause(1);
   
	RS = 1;                         	// return to character mode
}

//********************************************************************************
//* WriteLCD - Write a character to the Hitachi HD44780 LCD display
//********************************************************************************

void WriteLCD(char c)
{

   byte LowNibble, HighNibble;

  
  
   // Handle any special LCD control characters here. 

   if (c == '\r') 
   {
   	// New Line   
   	LCDLine++;
   	LCDLine = LCDLine % 4;
   	lcd_goto(LCDLine, LCDColumn);
   }
   else
   if (c >= 32)
   {
   
		// Write an ASCII character to the LCD 

   	HighNibble = (c >> 4) & 0x0F;
		PORTA =  HighNibble ;            // Output high nibble (leave other Port A bits alone)

		pulsout(E, 1);                    	// strobe the Enable line
	
   	//pause(1);
   	LowNibble = c & 0x0F;
		PORTA = LowNibble;             // output low nibble (leave other Port A bits alone)

		pulsout(E, 1);
   	//pause(1);
   }

	// Wait 1ms between characters for pacing
  	pause(1);  
   
}


//********************************************************************************
//* lcd_putc - Print a single byte to the LCD screen
//*
//* Inverts the bits as the serial LCD expects inverse polarity.
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: character c: 0 <= c <= 255
//* Return Value: none
//*
//* Also uses code from Peter H. Anderson
//********************************************************************************

void lcd_putc(char c) 
{
  	// Send character directly to the LCD display  
  
	WriteLCD(c);
 
  
}

//********************************************************************************
//*
//* lcd_clear - Clear the LCD screen
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: none
//* Return Value: none
//********************************************************************************

void lcd_clear(void) 
{

  	LCDLine = 0; 							// Current LCD Line
  	LCDColumn = 0; 							// Current LCD Column

 
  	// Send character directly to the LCD display 
  	
  WriteLCDCommand(LCDCLS);

 	// Wait 1ms between characters for pacing
  	pause(1);  
    
 
}

//********************************************************************************
//* 
//* lcd_home - Home the LCD cursor
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: none
//* Return Value: none
//********************************************************************************

void lcd_home(void) 
{
  	LCDLine = 0; 							// Current LCD Line
  	LCDColumn = 0; 							// Current LCD Column

  
  	// Send Home control character directly to the LCD display 
  	
  WriteLCDCommand(LCDHOME);

	// Wait 1ms between characters for pacing
  	pause(1);  
    
  
}

//********************************************************************************
//* lcd_puts - Print a string to the LCD screen
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: s: address of a string of bytes in DATA memory
//*               string must be NULL terminated
//* Returns: number of bytes written (number of chars printed)
//********************************************************************************

void lcd_puts(char *s) 
{   
 
		// Send the string to the LCD Display

		do
  		{  
			// Send byte to the LCD display  

 			WriteLCD(*s);                                // Write character directly to the LCD
 
  		} while( *s++);
  	
 
}

//********************************************************************************
//* lcd_printdec - Print a value in decimal to the LCD screen
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Note: output is always 3 digits: 001 not 1, 025, not 25
//* Arguments: x: value to print 0 <= x <= 255
//* Returns: none
//********************************************************************************

void lcd_printdec(byte x) 
{

  	byte count;
  
  	/* how many hundreds? */

  	count=0;
  	while(x > 99) 
	{
    	x = x - 100;
    	count++;
  	}

  	lcd_putc('0' + count);

  	/* how many tens? */

  	count=0;
  	while(x > 9) 
	{
    	x = x - 10;
    	count++;
  	}
  	lcd_putc('0' + count);

  	/* and finally units */

  	lcd_putc('0' + x);


}

//********************************************************************************
//*
//* lcd_printhex - Print a value in hex to the LCD screen
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: x: value to print 0 <= x <= 255
//* Returns: none
//********************************************************************************

void lcd_printhex(byte x) 
{

	// Convert to a 2 digit hex number

	//hex2(x);  // dfr

}

//********************************************************************************
//*
//* lcd_printbin - Print a value to the LCD screen as binary
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: x: value to print 0 <= x <= 255
//* Returns: none
//********************************************************************************

void lcd_printbin(char x) 
{

  	char bits[] = {128, 64, 32, 16, 8, 4, 2, 1};
  	char i = 0;
  	while(i < 8) 
	{
    	if(x & bits[i]) 
		{
      	lcd_putc('1');
    	} 
		else 
		{
      	lcd_putc('0');
    	}
    	i++;
  	}
}

//********************************************************************************
//*
//* lcd_goto - Move the cursor to the specified position
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: row: row on LCD: 0 <= row <= 1
//*            col: column on LCD: 0 <= col <= 16
//*
//*** Remember: columns are indexed from the left starting at col 0.
//* If you print off the screen, it is not displayed, but is stored in
//* the LCD's RAM. Use the lcd_scroll_left and lcd_scroll_right commands
//* to access the data
//*
//* Returns: nothing
//********************************************************************************

void lcd_goto(char row, char col) 
{
 
  	// Send Goto control character directly to the LCD display 
    WriteLCDCommand(LCDLINE1BASE + (LCDLINEINCR * row) + col);
 
    // Wait for command to complete
    pause(1);


}

//********************************************************************************
//*
//* lcd_scroll_left - Scroll the display N characters to the left
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//* Arguments: N: number of characters to scroll left. N > 0
//* There is a 10ms delay between each scroll action
//* Returns: nothing
//********************************************************************************

void lcd_scroll_left(char n) 
{

  	while(n) 
	{
	
  		// Send Scroll Left control character directly to the LCD display 
  	
 	 WriteLCDCommand(LCDSCRLLEFT);

	
    	pause(1);
    	n--;
  	}
}

//********************************************************************************
//*
//* lcd_scroll_right - Scroll the display N characters to the right
//*
//*** Assumes 9600 8N1 communication
//*** The constant LCDPIN defines which pin the LCD is on
//*** The constant LCDPORT defines which port the LCD in on
//*
//*  Arguments: N: number of characters to scroll right. N > 0
//* There is a 10ms delay between each scroll action
//* Returns: nothing
//********************************************************************************

void lcd_scroll_right(char n) 
{

  	while(n) 
	{


  		// Send Scroll Right control character directly to the LCD display 
  	
 	 WriteLCDCommand(LCDSCRLRIGHT);

    	 	
    	pause(1);
    	n--;
  	}
}

#ifdef LCD_TEST_1

void lcd_test_1(void)
{
	int i;
	
  	pause(1);
/*
  	while(1) 
	{
*/
   	for(i=32; i < 128; i++) 
		{
      	lcd_putc(i);
	    	lcd_putc(' ');
	    	lcd_printdec(i);
	    	lcd_putc(' ');
         lcd_printhex(i);
      	pause(100);
      	lcd_clear();				// clear LCD, home cursor
    	}
//  	}

}
#endif

#ifdef LCD_TEST_2

void lcd_test_2(void)
{
	int i;
   byte LineOffset;                       // Offset to second line of the LCD Display memory
	
// Display two test messages using LCD lines 1 and 2


    
   lcd_clear();						// clear LCD, home cursor
 
	pause(1000);

  // Write first message to line 1 of the LCD Display

   for (i=0; i< strlen(Msg1); i++)
	{  

	   // get character from EEPROM or RAM

	  	lcd_putc(Msg1[i]);                   	// write it

	}

	pause(2000);                    		// wait 2 seconds


	LineOffset = DDRam + Line2;						// move to line 2
	WriteLCDCommand(LineOffset);             		// write it

 
   // Write second message to line 2 of the LCD Display

   for (i=0; i< strlen(Msg2); i++)
	{  

		// get character from EEPROM

	  	lcd_putc(Msg2[i]);                   	// write it

	}

	pause(2000);                    		// wait 2 seconds

 
}
#endif

#ifdef LCD_TEST_3

/*******************************************************************************
 * lcdtest3.c
 *
 * Test lcd.lib and delays.lib
 * 
 * Shows the operation of lcd_scroll_left and lcd_scroll_right
 * commands
 *
 * Prints out the first 32 characters of the ASCII character set. Only 
 * the first 16 show on my LCD, so I scroll left by 16
 * characters. Then I scroll back right by 16 characters.
 *
 * Connect the LCD to port A pin 1 (i.e. physical pin 18 on a 16C84)
 *
 * Mark Crosbie 9/20/98
 ********************************************************************************
 */
 
void lcd_test_3 (void) 
{

  char i;

 
  pause(1000);

  lcd_clear();
  pause(1000);
  while(1) {
    for(i=32; i < 64; i++) {
            lcd_putc(i);
    }
    pause(1000);
    for(i=0; i < 16; i++) {
      lcd_scroll_left(1);
      pause(200);
    }
    pause(2000);
    for(i=0; i < 16; i++) {
      lcd_scroll_right(1);
      pause(200);
    }
    pause(2000);
    lcd_clear();
    
  }
}

#endif

#ifdef LCD_TEST_4

/*
 * lcdtest4.c
 *
 * Test lcd.lib and delays.lib
 *
 * Show the operation of lcd_puts
 *
 * Connect the LCD to port A pin 1 (i.e. physical pin 18 on a 16C84)
 *
 * Mark Crosbie 9/20/98
 */
 
 void lcd_test_4(void) 
 {

  char i;
  char s;

  
  pause(1000);

  lcd_clear();
  pause(1000);
  s = string;
  while(1) 
  {
    lcd_puts(s);
    pause(2000);
    lcd_clear();
  }
 
}

#endif

//****************************************************************************************
//* Converts a byte to an ascii string of 2 hex characters and writes                                                          
//* them to the USART.
//****************************************************************************************

int 		hex2(byte Value)
{   
    	HexByte_T HexByte;
 
 	// Convert binary value to an 8-Bit unsigned hex byte

        //btoh(Value, HexByte);  // dfr
 
        string((char *) HexByte);
 

	return TRUE;
}


//******************************************************************************
//* string - Writes a string terminated with a null character from RAM to the
//* serial port.                               
//******************************************************************************

void  string(char *s)
{
	// Send the string to the serial port
 
 
 
 	if (LCD_Ready)
	{
	
		// Send the string 

		do
  		{  
			// Send byte to the LCD display  

 			WriteLCD(*s);                                // Write character directly to the LCD
 
  		} while( *s++);
  	}
  	
 
 
}


/******************************************************************************
 * putchar - Send character to RS-232 port instead of stream (C40 version).
 ******************************************************************************/
int putchar(int c)
{

   if (c != '\n')
   {
     send_byte((byte) c);
   }
   else
   {
     send_byte((byte) CR);
     send_byte((byte) LF);
   } 
  return(TRUE);

}
//******************************************************************************
//* send_byte - Send character to RS-232 Port (USART)
//******************************************************************************
void            send_byte(byte c)
{
	byte Echo = ' ';

 
   	// Write data to USART

   	//Write_Byte_To_Uart((int) c);

 
 

}

//******************************************************************************
//* pulsout - Clock the data to the LCD
//******************************************************************************
void pulsout ( int Pin, int Time )
{
	E = 1;
	pause(Time);
    E = 0;
}
 
